@@ -1,15 +1,18 @@
 import numpy as np
 from difflib import get_close_matches
-import matplotlib.pyplot as plt
-import base64
-import re
-from io import BytesIO
+import plotly.graph_objects as go
 from .df_processers import (
     calculate_1rm,
     highest_weight_per_rep,
     estimate_weight_from_1rm,
     df_next_pareto,
 )
+from .plot_utils import (
+    slugify,
+    plotly_figure_to_html_div,
+    get_plotly_cdn_html,
+    get_plotly_preconnect_html,
+)
 
 
 def get_closest_exercise(df, Exercise):
@@ -21,85 +24,138 @@ def get_closest_exercise(df, Exercise):
         raise ValueError(f"No close match found for '{Exercise}'.")
 
 
-def plot_df(df, df_pareto=None, df_targets=None, Exercise: str = None):
-    df = df[df["Reps"] != 0]
+def plot_df(df_pareto=None, df_targets=None, Exercise: str = None):
+    if df_pareto is None or df_pareto.empty:
+        raise ValueError("df_pareto must be provided and non-empty")
 
     if Exercise is None:
-        exercises = df["Exercise"].unique()
-        fig, ax = plt.subplots()
-        for exercise in exercises:
-            exercise_df = df[df["Exercise"] == exercise]
-            ax.scatter(
-                exercise_df["Reps"] / max(exercise_df["Reps"]),
-                exercise_df["Weight"] / max(exercise_df["Weight"]),
-                label=exercise,
-            )
-        ax.set_title("Weight vs. Reps for All Exercises")
-        ax.set_xlabel("Reps")
-        ax.set_ylabel("Weight")
-        return fig
+        raise ValueError("Exercise must be specified")
 
-    closest_match = get_closest_exercise(df, Exercise)
-    df = df[df["Exercise"] == closest_match]
-    if df_pareto is not None:
-        df_pareto = df_pareto[df_pareto["Exercise"] == closest_match]
+    closest_match = get_closest_exercise(df_pareto, Exercise)
+    df_pareto = df_pareto[df_pareto["Exercise"] == closest_match]
     if df_targets is not None:
         df_targets = df_targets[df_targets["Exercise"] == closest_match]
+
+    rep_series = [df_pareto["Reps"]]
     if df_targets is not None and not df_targets.empty:
-        min_rep = df_targets["Reps"].min()
-        max_rep = df_targets["Reps"].max()
-    else:
-        min_rep = df["Reps"].min()
-        max_rep = df["Reps"].max()
+        rep_series.append(df_targets["Reps"])
 
-    fig, ax = plt.subplots()
+    min_rep = min(series.min() for series in rep_series)
+    max_rep = max(series.max() for series in rep_series)
+    plot_max_rep = max_rep + 1
 
-    if df_pareto is not None:
+    fig = go.Figure()
+
+    if df_pareto is not None and not df_pareto.empty:
         pareto_points = list(zip(df_pareto["Reps"], df_pareto["Weight"]))
         pareto_reps, pareto_weights = zip(*sorted(pareto_points, key=lambda x: x[0]))
+        pareto_reps = list(pareto_reps)
+        pareto_weights = list(pareto_weights)
 
         # Compute best 1RM from Pareto front
         one_rms = [calculate_1rm(w, r) for w, r in zip(pareto_weights, pareto_reps)]
         max_1rm = max(one_rms)
 
         # Generate dotted Epley decay line
-        x_vals = np.linspace(min_rep, max_rep, 10)
+        x_vals = np.linspace(min_rep, plot_max_rep, 100)
         y_vals = [estimate_weight_from_1rm(max_1rm, r) for r in x_vals]
-        ax.plot(x_vals, y_vals, "k--", label="Max Achieved 1RM", alpha=0.7)
-
-        ax.step(
-            pareto_reps, pareto_weights, color="red", marker="o", label="Pareto Front"
+        fig.add_trace(
+            go.Scatter(
+                x=x_vals,
+                y=y_vals,
+                mode="lines",
+                name="Max Achieved 1RM",
+                line=dict(color="black", dash="dash", width=2),
+                opacity=0.7,
+                hovertemplate="<b>Max 1RM Curve</b><br>"
+                + "Reps: %{x}<br>"
+                + "Weight: %{y:.1f} lbs<br>"
+                + f"1RM: {max_1rm:.1f}<extra></extra>",
+            )
         )
 
-    if df_targets is not None:
+        # Pareto step line
+        fig.add_trace(
+            go.Scatter(
+                x=pareto_reps,
+                y=pareto_weights,
+                mode="lines",
+                name="Pareto Front",
+                line=dict(color="red", shape="hv", width=2),
+                hovertemplate="<b>Pareto Front</b><extra></extra>",
+            )
+        )
+
+        # Pareto markers
+        fig.add_trace(
+            go.Scatter(
+                x=pareto_reps,
+                y=pareto_weights,
+                mode="markers",
+                name="Pareto Points",
+                marker=dict(color="red", size=10, symbol="circle"),
+                hovertemplate="<b>Pareto Point</b><br>"
+                + "Reps: %{x}<br>"
+                + "Weight: %{y} lbs<br>"
+                + "1RM: %{customdata:.1f}<extra></extra>",
+                customdata=one_rms,
+                showlegend=False,
+            )
+        )
+
+    if df_targets is not None and not df_targets.empty:
         target_points = list(zip(df_targets["Reps"], df_targets["Weight"]))
         target_reps, target_weights = zip(*sorted(target_points, key=lambda x: x[0]))
 
-        # Compute best 1RM from Pareto front
+        # Compute best 1RM from targets
         one_rms = [calculate_1rm(w, r) for w, r in zip(target_weights, target_reps)]
         min_1rm = min(one_rms)
 
-        # Generate dotted Epley decay line
-        x_vals = np.linspace(min_rep, max_rep, 10)
+        # Generate dotted Epley decay line for targets
+        x_vals = np.linspace(min_rep, plot_max_rep, 100)
         y_vals = [estimate_weight_from_1rm(min_1rm, r) for r in x_vals]
-        ax.plot(x_vals, y_vals, "g-.", label="Min Target 1RM", alpha=0.7)
-
-        ax.scatter(
-            df_targets["Reps"],
-            df_targets["Weight"],
-            color="green",
-            marker="x",
-            label="Targets",
+        fig.add_trace(
+            go.Scatter(
+                x=x_vals,
+                y=y_vals,
+                mode="lines",
+                name="Min Target 1RM",
+                line=dict(color="green", dash="dashdot", width=2),
+                opacity=0.7,
+                hovertemplate="<b>Target 1RM Curve</b><br>"
+                + "Reps: %{x}<br>"
+                + "Weight: %{y:.1f} lbs<br>"
+                + f"1RM: {min_1rm:.1f}<extra></extra>",
+            )
         )
 
-    # Plotting
-    ax.scatter(df["Reps"], df["Weight"], label="Data Points")
+        # Target markers
+        target_one_rms = [
+            calculate_1rm(w, r) for w, r in zip(target_weights, target_reps)
+        ]
+        fig.add_trace(
+            go.Scatter(
+                x=target_reps,
+                y=target_weights,
+                mode="markers",
+                name="Targets",
+                marker=dict(color="green", size=12, symbol="x"),
+                hovertemplate="<b>Target</b><br>"
+                + "Reps: %{x}<br>"
+                + "Weight: %{y} lbs<br>"
+                + "1RM: %{customdata:.1f}<extra></extra>",
+                customdata=target_one_rms,
+            )
+        )
 
-    ax.set_title(f"Weight vs. Reps for {closest_match}")
-    ax.set_xlabel("Reps")
-    ax.set_xlim(left=0, right=max_rep)
-    ax.set_ylabel("Weight")
-    ax.legend()
+    fig.update_layout(
+        title=f"Weight vs. Reps for {closest_match}",
+        xaxis_title="Reps",
+        yaxis_title="Weight (lbs)",
+        xaxis=dict(range=[0, plot_max_rep]),
+        hovermode="closest",
+        template="plotly_white",
+    )
 
     return fig
 
@@ -165,32 +221,18 @@ def render_table_fragment(df) -> str:
 
     figures_html: dict[str, str] = {}
 
-    def slugify(name: str) -> str:
-        """Return a normalized slug for the given exercise name."""
-        slug = re.sub(r"[^\w]+", "_", name)
-        slug = re.sub(r"_+", "_", slug).strip("_")
-        return slug.lower()
-
-    exercise_slug = {ex: slugify(ex) for ex in df["Exercise"].unique()}
+    exercise_slug = {ex: slugify(ex) for ex in df_records["Exercise"].unique()}
 
     for exercise, slug in exercise_slug.items():
-        fig = plot_df(df, df_records, df_targets, Exercise=exercise)
-        buf = BytesIO()
-        fig.savefig(buf, format="png", bbox_inches="tight")
-        buf.seek(0)
-        base64_img = base64.b64encode(buf.read()).decode("utf-8")
-        img_html = (
-            f'<img src="data:image/png;base64,{base64_img}" '
-            f'id="fig-{slug}" class="exercise-figure" '
-            'style="display:none; max-width:100%; height:auto;">'
-        )
+        fig = plot_df(df_records, df_targets, Exercise=exercise)
+        # Convert Plotly figure to HTML div with wrapper
+        img_html = plotly_figure_to_html_div(fig, slug, display="none")
         figures_html[exercise] = img_html
-        plt.close(fig)
 
     all_figures_html = "\n".join(figures_html.values())
 
     exercise_column = "Exercise"  # Adjust if needed
-    exercise_options = sorted(df[exercise_column].dropna().unique())
+    exercise_options = sorted(df_records[exercise_column].dropna().unique())
 
     dropdown_html = """
     <label for="exerciseDropdown">Filter by Exercise:</label>
@@ -230,10 +272,21 @@ def gen_html_viewer(df, *, embed_assets: bool = True) -> str:
     if not embed_assets:
         return fragment
 
-    js_and_css = """
+    js_and_css = (
+        """
+    <!-- Preconnect to CDNs for faster loading -->
+    <link rel="preconnect" href="https://code.jquery.com">
+    <link rel="preconnect" href="https://cdn.datatables.net">
+    <link rel="preconnect" href="https://cdn.jsdelivr.net">
+    """
+        + get_plotly_preconnect_html()
+        + "\n"
+        + get_plotly_cdn_html()
+        + """
+
     <!-- DataTables -->
     <link rel="stylesheet" href="https://cdn.datatables.net/1.13.4/css/jquery.dataTables.min.css"/>
-    <script src="https://code.jquery.com/jquery-3.5.1.js"></script>
+    <script src="https://code.jquery.com/jquery-3.7.1.min.js"></script>
     <script src="https://cdn.datatables.net/1.13.4/js/jquery.dataTables.min.js"></script>
 
     <!-- Select2 for searchable dropdown -->
@@ -243,37 +296,59 @@ def gen_html_viewer(df, *, embed_assets: bool = True) -> str:
     <!-- Custom Styling for Mobile -->
     <style>
     :root {
-        --bg: #ffffff;
-        --fg: #000000;
-        --bg-alt: #f5f5f5;
-        --border: #ccc;
+        --bg: #fafafa;
+        --fg: #1a1a1a;
+        --bg-alt: #ffffff;
+        --border: #e5e7eb;
+        --primary: #3b82f6;
+        --primary-hover: #2563eb;
+        --success: #10b981;
+        --shadow: 0 1px 3px 0 rgb(0 0 0 / 0.1);
     }
     @media (prefers-color-scheme: dark) {
         :root {
-            --bg: #121212;
-            --fg: #e0e0e0;
-            --bg-alt: #1e1e1e;
-            --border: #333;
+            --bg: #0f0f0f;
+            --fg: #e5e5e5;
+            --bg-alt: #1a1a1a;
+            --border: #2a2a2a;
+            --primary: #60a5fa;
+            --primary-hover: #3b82f6;
+            --success: #34d399;
+            --shadow: 0 1px 3px 0 rgb(0 0 0 / 0.3);
         }
     }
     [data-theme="dark"] {
-        --bg: #121212;
-        --fg: #e0e0e0;
-        --bg-alt: #1e1e1e;
-        --border: #333;
+        --bg: #0f0f0f;
+        --fg: #e5e5e5;
+        --bg-alt: #1a1a1a;
+        --border: #2a2a2a;
+        --primary: #60a5fa;
+        --primary-hover: #3b82f6;
+        --success: #34d399;
+        --shadow: 0 1px 3px 0 rgb(0 0 0 / 0.3);
     }
     [data-theme="light"] {
-        --bg: #ffffff;
-        --fg: #000000;
-        --bg-alt: #f5f5f5;
-        --border: #ccc;
+        --bg: #fafafa;
+        --fg: #1a1a1a;
+        --bg-alt: #ffffff;
+        --border: #e5e7eb;
+        --primary: #3b82f6;
+        --primary-hover: #2563eb;
+        --success: #10b981;
+        --shadow: 0 1px 3px 0 rgb(0 0 0 / 0.1);
     }
+
+    * {
+        transition: background-color 0.2s ease, border-color 0.2s ease, color 0.2s ease;
+    }
+
     body {
-        font-family: Arial, sans-serif;
+        font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif;
         font-size: 34px;
         padding: 28px;
         background-color: var(--bg);
         color: var(--fg);
+        line-height: 1.5;
     }
 
     table.dataTable {
@@ -283,18 +358,51 @@ def gen_html_viewer(df, *, embed_assets: bool = True) -> str:
         background-color: var(--bg-alt);
         color: var(--fg);
         border: 1px solid var(--border);
+        border-radius: 8px;
+        overflow: hidden;
+        box-shadow: var(--shadow);
+    }
+
+    table.dataTable thead th {
+        background-color: var(--bg);
+        font-weight: 600;
+        padding: 12px;
+        border-bottom: 2px solid var(--border);
+    }
+
+    table.dataTable tbody td {
+        padding: 10px 12px;
+    }
+
+    table.dataTable tbody tr {
+        border-bottom: 1px solid var(--border);
+    }
+
+    table.dataTable tbody tr:hover {
+        background-color: var(--bg);
     }
 
     label {
         font-size: 34px;
         color: var(--fg);
+        font-weight: 500;
+        margin-bottom: 8px;
+        display: inline-block;
     }
 
     select {
         font-size: 34px;
         color: var(--fg);
         background-color: var(--bg-alt);
-        border: 1px solid var(--border);
+        border: 2px solid var(--border);
+        border-radius: 6px;
+        padding: 8px 12px;
+    }
+
+    select:focus {
+        outline: none;
+        border-color: var(--primary);
+        box-shadow: 0 0 0 3px rgba(59, 130, 246, 0.1);
     }
 
     /* Dark mode overrides for DataTables and Select2 */
@@ -380,36 +488,102 @@ def gen_html_viewer(df, *, embed_assets: bool = True) -> str:
     }
 
     #uploadButton {
-        padding: 8px 16px;
+        padding: 10px 20px;
         border: none;
-        border-radius: 6px;
-        background-color: #007bff;
-        color: #fff;
+        border-radius: 8px;
+        background-color: var(--primary);
+        color: #ffffff;
         cursor: pointer;
+        font-weight: 600;
+        font-size: 28px;
+        box-shadow: var(--shadow);
+        transition: all 0.2s ease;
     }
 
     #uploadButton:hover {
-        background-color: #0056b3;
+        background-color: var(--primary-hover);
+        transform: translateY(-1px);
+        box-shadow: 0 4px 6px rgba(0, 0, 0, 0.15);
+    }
+
+    #uploadButton:active {
+        transform: translateY(0);
     }
 
     #csvFile {
-        padding: 6px;
-        border: 1px solid var(--border);
-        border-radius: 4px;
-        background-color: var(--bg);
+        padding: 10px;
+        border: 2px solid var(--border);
+        border-radius: 6px;
+        background-color: var(--bg-alt);
         color: var(--fg);
+        font-size: 28px;
+    }
+
+    #csvFile:focus {
+        outline: none;
+        border-color: var(--primary);
+        box-shadow: 0 0 0 3px rgba(59, 130, 246, 0.1);
+    }
+
+    .exercise-figure {
+        border-radius: 8px;
+        box-shadow: var(--shadow);
+        margin: 20px 0;
+        opacity: 0;
+        animation: fadeIn 0.3s ease-in forwards;
+    }
+
+    @keyframes fadeIn {
+        from {
+            opacity: 0;
+            transform: translateY(10px);
+        }
+        to {
+            opacity: 1;
+            transform: translateY(0);
+        }
+    }
+
+    .exercise-figure svg {
+        max-width: 100%;
+        height: auto;
+        display: block;
     }
 
     @media only screen and (max-width: 600px) {
-        table, thead, tbody, th, td, tr {
-            display: block;
+        body {
+            padding: 16px;
         }
-        th {
-            text-align: left;
+
+        table.dataTable {
+            font-size: 28px;
+        }
+
+        label {
+            font-size: 30px;
+        }
+
+        select {
+            font-size: 30px;
+        }
+
+        #uploadButton {
+            font-size: 26px;
+            padding: 12px 20px;
+        }
+
+        #csvFile {
+            font-size: 26px;
+        }
+
+        .upload-controls {
+            flex-direction: column;
+            align-items: stretch;
         }
     }
     </style>
     """
+    )
 
     upload_html = """
     <div class="upload-controls">
@@ -420,23 +594,46 @@ def gen_html_viewer(df, *, embed_assets: bool = True) -> str:
     """
 
     scripts = """
-    <script src="https://cdn.jsdelivr.net/pyodide/v0.24.1/full/pyodide.js"></script>
+    <script src="https://cdn.jsdelivr.net/pyodide/v0.24.1/full/pyodide.js" defer></script>
     <script type="module" src="main.js"></script>
     """
-    meta = '<meta charset="utf-8">'
+    meta = """
+    <meta charset="utf-8">
+    <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=5.0">
+    <meta name="description" content="KaiserLift workout analysis - Data-driven progressive overload with Pareto optimization">
+    """
     head_html = meta + js_and_css + scripts
     toggle_html = (
         '<button id="themeToggle" '
-        'style="position:fixed;top:10px;right:10px;padding:4px;'
+        'style="position:fixed;top:16px;right:16px;'
+        "padding:10px 14px;"
         "background:var(--bg-alt);color:var(--fg);"
-        'border:1px solid var(--border);border-radius:4px;">🌓</button>'
+        "border:2px solid var(--border);"
+        "border-radius:8px;"
+        "cursor:pointer;"
+        "font-size:24px;"
+        "box-shadow:var(--shadow);"
+        "transition:all 0.2s ease;"
+        'z-index:1000;"'
+        "onmouseover=\"this.style.transform='translateY(-2px)';this.style.boxShadow='0 4px 6px rgba(0,0,0,0.15)'\" "
+        "onmouseout=\"this.style.transform='translateY(0)';this.style.boxShadow='0 1px 3px rgba(0,0,0,0.1)'\">"
+        "🌓</button>"
     )
     theme_script = """
     <script>
-    document.getElementById('themeToggle').addEventListener('click', () => {
-      const root = document.documentElement;
-      const current = root.getAttribute('data-theme') === 'dark' ? 'light' : 'dark';
-      root.setAttribute('data-theme', current);
+    // Wait for DOM to be ready
+    document.addEventListener('DOMContentLoaded', function() {
+      // Theme toggle
+      const themeToggleBtn = document.getElementById('themeToggle');
+      if (themeToggleBtn) {
+        themeToggleBtn.addEventListener('click', () => {
+          const root = document.documentElement;
+          const current = root.getAttribute('data-theme') === 'dark' ? 'light' : 'dark';
+          root.setAttribute('data-theme', current);
+        });
+      }
+      // Note: DataTable and Select2 initialization is handled by main.js
+      // to avoid duplicate initialization
     });
     </script>
     """
