@@ -21,9 +21,16 @@ def test_pipeline_fragment_without_assets() -> None:
     with csv_path.open("rb") as fh:
         html = pipeline([fh], embed_assets=False)
     assert "<table" in html
-    assert "<script" not in html
-    assert "<link" not in html
+    # Should not have external CDN scripts/styles
+    assert "<script src=" not in html
+    assert "<link href=" not in html
+    # Should not have upload controls
     assert 'id="uploadButton"' not in html
     assert 'id="csvFile"' not in html
     assert 'id="result"' not in html
     assert 'id="uploadProgress"' not in html
+    # Should not have full HTML page structure
+    assert "<html>" not in html
+    assert "<head>" not in html
+    assert "<body>" not in html
+    # Inline scripts for Plotly initialization are OK
